# coding: utf-8

sdvar_logger = 'sunday_logger'
sdvar_loggerid = 'loggerId'
sdvar_premust = 'sunday_premust'
sdvar_exectime = 'sunday_exectime'
sdvar_exception = 'sunday_exception'
sdvar_loglevel = 'sunday_loglevel'
